LRT = LibStub("AceAddon-3.0"):NewAddon("LessrabiTimer", "AceConsole-3.0", "AceEvent-3.0")

-- TBD:
--
-- Submission checklist
-- comment out all Debug Print statements!
-- update toc file revision
local TimeSinceLastUpdate = nil
local LRT_UpdateInterval = .05
local showDebug = false
local showTrace = false
local LRTStatusBar = nil
local LRTTarget = "Moorabi"
--
-- This function returns a legit string no matter what you pass it.
-- It gets used in debug messages to avoid annoying lua errors.
function LRT:NS(arg1)
	if arg1 == nil then
		return "nil"
	elseif type(arg1) == "table" then
		local myval = "{"
		for i, j in pairs(arg1) do
			myval = myval.." ["..LRT:NS(i).."]="..LRT:NS(j)..","
		end
		myval = myval.."}"
		return myval
	else
		return tostring(arg1)
	end
end

function LRT:DebugTrace (arg1, enter)
	if showTrace == true then
		if (enter) then
			DEFAULT_CHAT_FRAME:AddMessage("LRT:"..arg1.." enter")
		else
			DEFAULT_CHAT_FRAME:AddMessage("LRT:"..arg1.." exit")
		end
	end
end

function LRT:DebugPrint (arg1)
	if showDebug == true then
		DEFAULT_CHAT_FRAME:AddMessage("LRT:"..arg1)
	end
end

function LRT:OnInitialize()
  -- Code that you want to run when the addon is first loaded goes here.
	LRT:DebugTrace ("OnInitialize", true)
	LRT:RegisterChatCommand("lrt", "SlashProcessorFunc")
	LRT:MakeWidgets()
	LRT:DebugTrace ("OnInitialize", false)
end


function LRT:OnEnable()
	LRT:DebugTrace ("OnEnable", true)
    -- Called when the addon is enabled
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED", "ProcessCombatLogEvent")
	LRT:DebugTrace ("OnEnable")
end

function LRT:OnDisable()
	LRT:DebugTrace ("OnDisable", true)
    -- Called when the addon is disabled
	self:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	LRT:DebugTrace ("OnDisable")
end

function LRT:SlashProcessorFunc(input)
	LRT:DebugTrace ("SlashProcessorFunc", true)
  -- Process the slash command ('input' contains whatever follows the slash command)
	if (input == "toggleDebug") then
		if showDebug then
			LRT:DebugPrint ("turning debug messages off")
			showDebug = false
		else
			showDebug = true
			LRT:DebugPrint ("turning debug messages on")
		end
	elseif (input == "toggleTrace") then
		if showTrace then
			LRT:DebugPrint ("turning trace messages off")
			showTrace = false
		else
			showTrace = true
			LRT:DebugPrint ("turning trace messages on")
		end
	elseif (input == "test") then
		LRTStatusBar.lastCast = GetTime()
		LRTStatusBar.interval = 10
		LRTStatusBar:Show()
	end
	LRT:DebugTrace ("SlashProcessorFunc")
end

-- update the bars
function LRT:OnUpdate (self, elapsed)
	if self:IsShown() then
		if self.TimeSinceLastUpdate == nil then 
			self.TimeSinceLastUpdate = elapsed
		end
		self.TimeSinceLastUpdate = self.TimeSinceLastUpdate + elapsed; 	
		if (self.TimeSinceLastUpdate > LRT_UpdateInterval) then
			local currentTime = GetTime()
			self.TimeSinceLastUpdate = 0;
			
			if self.lastCast and self.interval then
				local remaining = self.lastCast + self.interval - currentTime
				if remaining > 0 then
					value = remaining / self.interval * 100
					self:SetValue (value)
					self.timer:SetFormattedText("%.1f", remaining)
				else
					self.lastCast = nil
					self.interval = nil
					self:Hide()
				end
			end
		end
	end
end

function LRT:MakeWidgets()
	LRT:DebugTrace ("MakeWidgets", true)

	LRTStatusBar = CreateFrame("StatusBar", nil, UIParent); 
	LRTStatusBar:SetMovable (true)
	LRTStatusBar:EnableMouse (true)
	LRTStatusBar:SetScript("OnMouseDown", LRTStatusBar.StartMoving)
	LRTStatusBar:SetScript("OnMouseUp", LRTStatusBar.StopMovingOrSizing)
	LRTStatusBar:SetWidth(200); 
	LRTStatusBar:SetHeight(20); 
	LRTStatusBar:SetPoint("CENTER", UIParent, "CENTER", 0, 50); 
	LRTStatusBar:SetStatusBarTexture("Interface\\TargetingFrame\\UI-StatusBar"); 
	LRTStatusBar:SetMinMaxValues(0, 100); 
	LRTStatusBar:SetValue(50);
	LRTStatusBar.timer = LRTStatusBar:CreateFontString(nil, nil, "GameFontNormalLarge")
	LRTStatusBar.timer:SetPoint("LEFT", LRTStatusBar, "LEFT", 10, 0)
	LRTStatusBar.timer:SetTextColor (1, 1, 0, 1)
	LRTStatusBar.title = LRTStatusBar:CreateFontString(nil, nil, "GameFontNormalSmall")
	LRTStatusBar.title:SetPoint("BOTTOM", LRTStatusBar, "TOP", 10, 0)
	LRTStatusBar.title:SetTextColor (1, 1, 0, 1)
	LRTStatusBar.title:SetText ("Next Transformation cast")
	LRTStatusBar:SetScript ("OnUpdate", function (self, elapsed) LRT:OnUpdate (self, elapsed) end)
	LRTStatusBar:Hide()
	LRT:DebugTrace ("MakeWidgets")
end

-- look for incoming dot ticks and set the tick marker to the time of the next calculated tick
function LRT:ProcessCombatLogEvent (_, timestamp, event, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, ...)
	-- only look at the player's events
	if UnitGUID("player") == srcGUID then
		-- only look at periodic spell damage
		if (string.find (event, "SPELL_INTERRUPT") and dstName == LRTTarget) then
			local spellName = select (2, ...)
			local interval
			local health = UnitHealth("target")/UnitHealthMax("target")
			if (health > .5) then
				interval = 10
			elseif (health > .35) then
				interval = 5
			else
				interval = 4
			end
			LRT:DebugPrint (LRT:NS(event).." "..LRT:NS(spellName).." "..LRT:NS(dstName).."H="..LRT:NS(health).." interval="..interval)
			LRTStatusBar.lastCast = GetTime()
			LRTStatusBar.interval = interval
			LRTStatusBar:Show()
--			LRT:DebugPrint (event.." "..spellName)
			-- loop over buttons
		end
	end
end